#include <gtkmm.h>
#include <libglademm.h>
#include "mywidget.h"
#include <iostream>

/** Initialise the widgets so the GTypes are known */

//This is meant to be like the wrap_new() functions that are found in the gtkmm.cc files and used in the generated wrap_init.cc file in gtkmm.
//We register this with Glib::wrap_register(), so that it is available to Glib::wrap(), which is used by Glade::get_widget().

static Glib::ObjectBase* mywidget_wrap_new(GObject* o)
{
  return manage(new MyWidget((GtkLabel*)(o)));
}

//Must be called after Gtk::Main or Gtk::Main::init_gtkmm_internals().
void init_widgets() {
  Gtk::Main::init_gtkmm_internals(); //Not really necessary if we have already called Gtk::Main.

  MyWidget* dummy = new MyWidget();
  GType gtype = G_OBJECT_TYPE(dummy->gobj());
  delete( dummy ); 

  Glib::wrap_register(gtype, &mywidget_wrap_new);
}

int main(int argc, char* argv[]) {
  Gtk::Widget* mywidget = 0L;
  Gtk::Window* mainwindow = 0L;
  Gtk::Main kit( argc,argv );

  // If commented out Glade does not know about our custom type.
  // If active, Glade does not produce that error anymore (so the
  // GType of the MyWidget is indeed registered) but a normal
  // Gtk::Label is constructed instead of the MyWidget.
  init_widgets();

  Glib::RefPtr<Gnome::Glade::Xml> gxml = Gnome::Glade::Xml::create("mywidget-example.glade");
  gxml->get_widget("mainwindow",mainwindow);
  gxml->get_widget("mywidget",mywidget);
  mainwindow->show();

  std::cout << "GType name of mywidget gobj(): " << G_OBJECT_TYPE_NAME(mywidget->gobj()) << std::endl;

  std::cout << "Type of Gtk::Label  : " << Gtk::Label::get_type() << std::endl;
  std::cout << "Type of MyWidget    : " << MyWidget::get_type() << std::endl;

  if( dynamic_cast<Gtk::Label*>(mywidget) )
    std::cout << "Found widget is of Gtk::Label type." << std::endl;
  else
    std::cout << "Found widget is NOT of Gtk::Label type." << std::endl;

  if( dynamic_cast<MyWidget*>(mywidget) )
    std::cout << "Found widget is of MyWidget type." << std::endl;
  else
    std::cout << "Found widget is NOT of MyWidget type." << std::endl;

  kit.run(*mainwindow);
  return 0;
}
